unit JSONHelperU;

interface

uses
  Classes, DBXJSON, Generics.Collections, DB;

type
  TJSONArrayEnumerator = class
  private
    FIndex: Integer;
    FArray: TJSONArray;
  public
    constructor Create(const AArray: TJSONArray);
    function GetCurrent: TJSONValue; inline;
    function MoveNext: Boolean;
    property Current: TJSONValue read GetCurrent;
  end;

  TJSONArrayHelper = class helper for TJSONArray
  public
    function GetEnumerator: TJSONArrayEnumerator;
    function AsStringList: TStringList;
    function AsIntList: TList<Integer>;
  end;

  TJSONDatasetHelper = class helper for TDataset
  public
    function ToJsonObject: TJSONObject;
    function ToJsonArray: TJSONArray;
    function AsJSONObjectString: String;
    function AsJSONArrayString: String;
  end;

function JSONStringToStringList(const JSONString: string): TStringList;
function JSONStringToIntList(const JSONString: string): TList<Integer>;

implementation

uses
  SysUtils, Variants;

{ TJSONArrayEnumerator }

constructor TJSONArrayEnumerator.Create(const AArray: TJSONArray);
begin
  inherited Create;
  FIndex := -1;
  FArray := AArray;
end;

function TJSONArrayEnumerator.GetCurrent: TJSONValue;
begin
  Result := FArray.Get(FIndex);
end;

function TJSONArrayEnumerator.MoveNext: Boolean;
begin
  Result := FIndex < FArray.Size - 1;
  if Result then
    Inc(FIndex);
end;

{ TJSONArrayHelper }

function TJSONArrayHelper.AsIntList: TList<Integer>;
var
  JSONValue: TJSONValue;
begin
  Result := TList<Integer>.Create;

  for JSONValue in Self do
    if JSONValue is TJSONString then
      Result.Add(StrToIntDef(TJSONString(JSONValue).Value, -1))
    else if JSONValue is TJSONObject then
      Result.Add(StrToIntDef(JSONValue.ToString, -1));
end;

function TJSONArrayHelper.AsStringList: TStringList;
var
  JSONValue: TJSONValue;
begin
  Result := TStringList.Create;

  for JSONValue in Self do
    if JSONValue is TJSONString then
      Result.Add(TJSONString(JSONValue).Value)
    else if JSONValue is TJSONObject then
      Result.Add(JSONValue.ToString);
end;

function TJSONArrayHelper.GetEnumerator: TJSONArrayEnumerator;
begin
  Result := TJSONArrayEnumerator.Create(Self);
end;

function JSONStringToIntList(const JSONString: string): TList<Integer>;
var
  JSONValue: TJSONValue;
begin
  Result := nil;

  if JSONString = '' then
    exit;

  JSONValue := TJSONObject.ParseJSONValue(JSONString);

  if JSONValue.Null then // Hvis ikke strengen er korrekt JSON formateret s er den Null
    exit;

  if JSONValue is TJSONArray then
    exit(TJSONArray(JSONValue).AsIntList);
end;

function JSONStringToStringList(const JSONString: string): TStringList;
var
  JSONValue: TJSONValue;
begin
  Result := nil;

  if JSONString = '' then
    exit;

  JSONValue := TJSONObject.ParseJSONValue(JSONString);

  if JSONValue.Null then // Hvis ikke strengen er korrekt JSON formateret s er den Null
    exit;

  if JSONValue is TJSONArray then
    exit(TJSONArray(JSONValue).AsStringList);
end;

{ TJSONDatasetHelper }

function TJSONDatasetHelper.AsJSONArrayString: String;
begin
  with ToJsonArray do
    try
      Result := ToString;
    finally
      free;
    end;
end;

function TJSONDatasetHelper.AsJSONObjectString: String;
begin
  with ToJsonObject do
    try
      Result := ToString;
    finally
      free;
    end;
end;

function TJSONDatasetHelper.ToJsonArray: TJSONArray;
begin
  Result := TJSONArray.Create;
  First;
  while not EOF do
  begin
    Result.AddElement(ToJsonObject);
    Next;
  end;
end;

function TJSONDatasetHelper.ToJsonObject: TJSONObject;
var
  DataField: TField;
  DataValue: TJSONValue;
begin
  if RecordCount = 0 then
    exit(nil);

  Result := TJSONObject.Create;
  for DataField in Fields do
    begin
      if VarIsType(DataField.Value, varInteger) then
        DataValue := TJSONNumber.Create(DataField.Value)
      else
        DataValue := TJSONString.Create(DataField.Value);
      Result.AddPair(DataField.FieldName, DataValue);
    end;
  Result.Owned := True;
end;

end.
