unit RESTHandlersU;

interface

uses
  SysUtils, RESTWebModuleU, Generics.Collections, HTTPApp, ADODB;

type
  TTestHandler = class sealed(TRESTRequestHandler)
  protected
    function GetPath: string; override; final;
    function HandleGet(Request: TWebRequest; Response: TWebResponse; Variables: TList<string>): Boolean; override; final;
  end;

  TDateTimeHandler = class sealed(TRESTRequestHandler)
  protected
    function GetPath: string; override; final;
    function HandleGet(Request: TWebRequest; Response: TWebResponse; Variables: TList<string>): Boolean; override; final;
  end;

  TAdminLogonRESTRequestHandler = class(TRESTRequestHandler)
  protected
    function GetPath: string; override;
    function HandleGet(Request: TWebRequest; Response: TWebResponse; Variables: TList<string>): Boolean; override;
  end;

  TAdminTestRESTRequestHandler = class(TRESTRequestHandler)
  protected
    function GetPath: string; override;
    function HandleGet(Request: TWebRequest; Response: TWebResponse; Variables: TList<string>): Boolean; override;
  end;

  TAdminUsersRESTRequestHandler = class(TRESTRequestHandler)
  protected
    function GetPath: string; override;
    function HandleGet(Request: TWebRequest; Response: TWebResponse; Variables: TList<string>): Boolean; override;
  end;

implementation

uses
  TestObjectU, ModelU, JSONHelperU;

{ TTestHandler }

function TTestHandler.GetPath: string;
begin
  Result := '/Test/*';
end;

function TTestHandler.HandleGet(Request: TWebRequest; Response: TWebResponse; Variables: TList<string>): Boolean;
var
  Buffer: string;
begin
  Result := True;
  Response.ContentType := CONTENT_TYPE_HTML;
  Response.StatusCode := HTTP_STATUS_OK;
  Response.Content := '<H1>Variabler medgivet:</H1>';
  if Variables.Count = 0 then
    Response.Content := Response.Content + 'Ingen'
  else
    for Buffer in Variables do
      Response.Content := Response.Content + Buffer + '</br>';

  Response.Content := Response.Content + '<H1><br>Felter medgivet:</H1>';

  for Buffer in Request.QueryFields do
    Response.Content := Response.Content + Buffer + '</br>';
end;

{ TDateTimeHandler }

function TDateTimeHandler.GetPath: string;
begin
  Result := '/DateTime*';
end;

function TDateTimeHandler.HandleGet(Request: TWebRequest; Response: TWebResponse; Variables: TList<string>): Boolean;
var
  TestObject: TMyTestObject;
begin
  TestObject := TMyTestObject.Create;
  try
    TestObject.Tid := now;
    TestObject.sTid := FormatDateTime(FormatSettings.LongDateFormat + #32 + FormatSettings.LongTimeFormat, TestObject.Tid);
    Response.ContentType := CONTENT_TYPE_JSON;
    Response.Content := TestObject.ToJsonString;
  finally
    FreeAndNil(TestObject);
  end;

  Result := True;
end;

{ TAdminLogonRESTRequestHandler }

function TAdminLogonRESTRequestHandler.GetPath: string;
begin
  Result := '/system/logon';
end;

function TAdminLogonRESTRequestHandler.HandleGet(Request: TWebRequest; Response: TWebResponse; Variables: TList<string>): Boolean;
var
  AuthenticationID: string;
begin
  AuthenticationID := Model.AdminLogon(Request.QueryFields.Values['username'], Request.QueryFields.Values['password']);
  if AuthenticationID <> EMPTY_GUID then
  begin
    Response.StatusCode := HTTP_STATUS_OK;
    Response.Content := AuthenticationID;
    Response.ContentType := CONTENT_TYPE_TEXT;
  end
  else
    Response.StatusCode := HTTP_STATUS_UNAUTHORIZED;

  Result := True;
end;

{ TAdminTestRESTRequestHandler }

function TAdminTestRESTRequestHandler.GetPath: string;
begin
  Result := '/system/admin/*/test';
end;

function TAdminTestRESTRequestHandler.HandleGet(Request: TWebRequest; Response: TWebResponse; Variables: TList<string>): Boolean;
begin
  Result := True;
  if IsAuthenticated(Variables[0], Response) then
  begin
    Response.StatusCode := HTTP_STATUS_OK;
    Response.Content := 'Hurrra!!! Vi er logger ind';
    Response.ContentType := CONTENT_TYPE_TEXT;;
  end
  else
    Response.StatusCode := HTTP_STATUS_UNAUTHORIZED;
end;

{ TAdminUsersRESTRequestHandler }

function TAdminUsersRESTRequestHandler.GetPath: string;
begin
  Result := '/system/admin/*/users';
end;

function TAdminUsersRESTRequestHandler.HandleGet(Request: TWebRequest; Response: TWebResponse; Variables: TList<string>): Boolean;
var
  Query: TADOQuery;
begin
  Result := True;
  if not IsAuthenticated(Variables[0], Response) then
  begin
    Response.StatusCode := HTTP_STATUS_UNAUTHORIZED;
    Exit;
  end;

  Response.StatusCode := HTTP_STATUS_OK;
  Response.ContentType := 'application/json';
  Response.Content := Model.AdminListUsers.AsJSONArrayString;
  FreeAndNil(Query);
end;

initialization

TRESTWebModule.RegisterHandler(TTestHandler);
TRESTWebModule.RegisterHandler(TDateTimeHandler);
TRESTWebModule.RegisterHandler(TAdminLogonRESTRequestHandler);
TRESTWebModule.RegisterHandler(TAdminTestRESTRequestHandler);
TRESTWebModule.RegisterHandler(TAdminUsersRESTRequestHandler);


end.
